==> japhar-0.04.tar.gz: japhar-0.04/include/interp.h <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   interp.h -- java interpreter definitions.
   Created: Chris Toshok <toshok@hungry.com>, 13-Aug-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#ifndef _interp_h
#define _interp_h

#include "config.h"
#include "objects.h"
#include "op_stack.h"
#include "sig.h"
#include "ClazzFile.h"

typedef struct _StackFrame StackFrame;

typedef struct JThreadInfo {
  /* A reference to the java thread which represents
     this native thread. */
  jobject java_thread;

  StackFrame* stack_highwater; /* the highest address possible for this thread's stack. */
  StackFrame* stack_lowwater;  /* the lowest address -- the address where the thread stack
				  is allocated. We use this to check for depth == 0 */
  StackFrame* top_frame;

  OpStack *op_stack; /* this thread's operand stack */

  char *name;
  int alive; /* true if thread is started and not terminated yet */

  JavaStackItem return_value;
  jint current_state;  /* whether we're suspended, running, etc, etc. */
} JThreadInfo;

typedef struct {
  char *key;
  char *value;
} propertySpec;

/* access macros */
#define STATE(t) (t)->current_state
#define TOPFRAME(t) (t)->top_frame
#define PRIORITY(t) (t)->priority
#define NAME(t) (t)->name

#define FRAME_NATIVE			0x01
#define FRAME_HASCLASSLOADER	0x02

struct _StackFrame {
  int frame_size;
  int depth;

  unsigned char flags;

  MethodStruct *method;
  JThreadInfo *thread_info;
  JNIEnv *jni_env;

  int pc;

  char *className;
  char *methodName;
  char *methodSig;

  JavaStackItem *opstack_top;

  jobject this_pointer;
  JavaStackItem vars[1];
};

/* access macros */

#define PARENT(f) (StackFrame*)(((char*)f) + f->frame_size)
#define ENV(f) (f)->jni_env
#define THREAD_INFO(f) (f)->thread_info
#define STACKDEPTH(f) (f)->depth

#define THIS(f) (f)->this_pointer

#define CLASSNAME(f) (f)->className
#define METHODNAME(f) (f)->methodName
#define METHODSIG(f) (f)->methodSig

#define METHOD(f) (f)->method
#define CODE(f) (f)->method->code
#define EXCEPTIONS(f) (f)->method->exceptions
#define NUMEXCEPTIONS(f) (f)->method->num_exception_blocks
#define PC(f) (f)->pc
#define VARS(f) (f)->vars

#define OPSTACK(f) THREAD_INFO(f)->op_stack
#define OPSTACK_TOP(f) (f)->opstack_top

StackFrame *create_frame_for_method(JThreadInfo *thr_info, MethodStruct *info);
void destroy_frame (StackFrame *f);

extern u1 wide_vindex2;
extern int wide_vindex_set;

void fill_local_vars_from_stack (StackFrame *df, /* destination frame */
				 int num_words_needed,
				 int static_p); /* is this a static call? */
void fill_local_vars (StackFrame *df, /* destinatin frame */
		      MethodStruct *method,
		      jvalue *args, /* must match the number in the signature. */
		      jobject obj); /* NULL for a static call. */

StackFrame* push_frame(JThreadInfo *thr_info, int num_vars);
void pop_frame(JThreadInfo *thr_info);
StackFrame *get_frame_parent(StackFrame *f);

void do_native_method_call(JNIEnv *env, MethodStruct *);

void execute_opcode(StackFrame *f, u1 opcode);

jvalue CallJavaMethod(MethodStruct *method, jobject obj, jvalue *args);
jvalue CallStaticJavaMethod(MethodStruct *method, jclass clazz, jvalue *args);

void interp_loop(StackFrame *current_frame);

void set_instance_field(obj_thunk *obj, FieldStruct *field, JavaStackItem value);
void set_static_field(ClazzFile *clazz, FieldStruct *field, JavaStackItem value);

void get_instance_field(obj_thunk *obj, FieldStruct *field, JavaStackItem *value);
void get_static_field(ClazzFile *clazz, FieldStruct *field, JavaStackItem *value);

#define get_next_byte(f) (CODE(f)[ PC(f) ++ ])

extern void maybe_enter_monitor_for_method(JNIEnv *env,
					   MethodStruct *method,
					   obj_thunk *this_obj);
extern void maybe_exit_monitor_for_method(JNIEnv *env,
					  MethodStruct *method,
					  obj_thunk *this_obj);
extern void maybe_push_return_value(StackFrame *f);

typedef void (*interp_function)(StackFrame *);

typedef struct opcode_struct {
  char name[26];
  int num_bytes_needed;
  int num_stack_items_before;
  int num_stack_items_after;
  interp_function func;
} opcode;

/* the interpreter functions. */
#define EXTERN_INTERP_FUNC(name) extern void op_##name(StackFrame *f)

EXTERN_INTERP_FUNC(nop);
EXTERN_INTERP_FUNC(aconst_null);
EXTERN_INTERP_FUNC(iconst_m1);
EXTERN_INTERP_FUNC(iconst_0);
EXTERN_INTERP_FUNC(iconst_1);
EXTERN_INTERP_FUNC(iconst_2);
EXTERN_INTERP_FUNC(iconst_3);
EXTERN_INTERP_FUNC(iconst_4);
EXTERN_INTERP_FUNC(iconst_5);
EXTERN_INTERP_FUNC(lconst_0);
EXTERN_INTERP_FUNC(lconst_1);
EXTERN_INTERP_FUNC(fconst_0);
EXTERN_INTERP_FUNC(fconst_1);
EXTERN_INTERP_FUNC(fconst_2);
EXTERN_INTERP_FUNC(dconst_0);
EXTERN_INTERP_FUNC(dconst_1);
EXTERN_INTERP_FUNC(bipush);
EXTERN_INTERP_FUNC(sipush);
EXTERN_INTERP_FUNC(ldc);
EXTERN_INTERP_FUNC(ldc_w);
EXTERN_INTERP_FUNC(ldc2w);
EXTERN_INTERP_FUNC(iload);
EXTERN_INTERP_FUNC(lload);
EXTERN_INTERP_FUNC(fload);
EXTERN_INTERP_FUNC(dload);
EXTERN_INTERP_FUNC(aload);
EXTERN_INTERP_FUNC(iload_0);
EXTERN_INTERP_FUNC(iload_1);
EXTERN_INTERP_FUNC(iload_2);
EXTERN_INTERP_FUNC(iload_3);
EXTERN_INTERP_FUNC(lload_0);
EXTERN_INTERP_FUNC(lload_1);
EXTERN_INTERP_FUNC(lload_2);
EXTERN_INTERP_FUNC(lload_3);
EXTERN_INTERP_FUNC(fload_0);
EXTERN_INTERP_FUNC(fload_1);
EXTERN_INTERP_FUNC(fload_2);
EXTERN_INTERP_FUNC(fload_3);
EXTERN_INTERP_FUNC(dload_0);
EXTERN_INTERP_FUNC(dload_1);
EXTERN_INTERP_FUNC(dload_2);
EXTERN_INTERP_FUNC(dload_3);
EXTERN_INTERP_FUNC(aload_0);
EXTERN_INTERP_FUNC(aload_1);
EXTERN_INTERP_FUNC(aload_2);
EXTERN_INTERP_FUNC(aload_3);
EXTERN_INTERP_FUNC(iaload);
EXTERN_INTERP_FUNC(laload);
EXTERN_INTERP_FUNC(faload);
EXTERN_INTERP_FUNC(daload);
EXTERN_INTERP_FUNC(aaload);
EXTERN_INTERP_FUNC(baload);
EXTERN_INTERP_FUNC(caload);
EXTERN_INTERP_FUNC(saload);
EXTERN_INTERP_FUNC(istore);
EXTERN_INTERP_FUNC(lstore);
EXTERN_INTERP_FUNC(fstore);
EXTERN_INTERP_FUNC(dstore);
EXTERN_INTERP_FUNC(astore);
EXTERN_INTERP_FUNC(istore_0);
EXTERN_INTERP_FUNC(istore_1);
EXTERN_INTERP_FUNC(istore_2);
EXTERN_INTERP_FUNC(istore_3);
EXTERN_INTERP_FUNC(lstore_0);
EXTERN_INTERP_FUNC(lstore_1);
EXTERN_INTERP_FUNC(lstore_2);
EXTERN_INTERP_FUNC(lstore_3);
EXTERN_INTERP_FUNC(fstore_0);
EXTERN_INTERP_FUNC(fstore_1);
EXTERN_INTERP_FUNC(fstore_2);
EXTERN_INTERP_FUNC(fstore_3);
EXTERN_INTERP_FUNC(dstore_0);
EXTERN_INTERP_FUNC(dstore_1);
EXTERN_INTERP_FUNC(dstore_2);
EXTERN_INTERP_FUNC(dstore_3);
EXTERN_INTERP_FUNC(astore_0);
EXTERN_INTERP_FUNC(astore_1);
EXTERN_INTERP_FUNC(astore_2);
EXTERN_INTERP_FUNC(astore_3);
EXTERN_INTERP_FUNC(iastore);
EXTERN_INTERP_FUNC(lastore);
EXTERN_INTERP_FUNC(fastore);
EXTERN_INTERP_FUNC(dastore);
EXTERN_INTERP_FUNC(aastore);
EXTERN_INTERP_FUNC(bastore);
EXTERN_INTERP_FUNC(castore);
EXTERN_INTERP_FUNC(sastore);
EXTERN_INTERP_FUNC(pop);
EXTERN_INTERP_FUNC(pop2);
EXTERN_INTERP_FUNC(dup);
EXTERN_INTERP_FUNC(dupx1);
EXTERN_INTERP_FUNC(dupx2);
EXTERN_INTERP_FUNC(dup2);
EXTERN_INTERP_FUNC(dup2x1);
EXTERN_INTERP_FUNC(dup2x2);
EXTERN_INTERP_FUNC(swap);
EXTERN_INTERP_FUNC(iadd);
EXTERN_INTERP_FUNC(ladd);
EXTERN_INTERP_FUNC(fadd);
EXTERN_INTERP_FUNC(dadd);
EXTERN_INTERP_FUNC(isub);
EXTERN_INTERP_FUNC(lsub);
EXTERN_INTERP_FUNC(fsub);
EXTERN_INTERP_FUNC(dsub);
EXTERN_INTERP_FUNC(imul);
EXTERN_INTERP_FUNC(lmul);
EXTERN_INTERP_FUNC(fmul);
EXTERN_INTERP_FUNC(dmul);
EXTERN_INTERP_FUNC(idiv);
EXTERN_INTERP_FUNC(ldiv);
EXTERN_INTERP_FUNC(fdiv);
EXTERN_INTERP_FUNC(ddiv);
EXTERN_INTERP_FUNC(irem);
EXTERN_INTERP_FUNC(lrem);
EXTERN_INTERP_FUNC(frem);
EXTERN_INTERP_FUNC(drem);
EXTERN_INTERP_FUNC(ineg);
EXTERN_INTERP_FUNC(lneg);
EXTERN_INTERP_FUNC(fneg);
EXTERN_INTERP_FUNC(dneg);
EXTERN_INTERP_FUNC(ishl);
EXTERN_INTERP_FUNC(lshl);
EXTERN_INTERP_FUNC(ishr);
EXTERN_INTERP_FUNC(lshr);
EXTERN_INTERP_FUNC(iushr);
EXTERN_INTERP_FUNC(lushr);
EXTERN_INTERP_FUNC(iand);
EXTERN_INTERP_FUNC(land);
EXTERN_INTERP_FUNC(ior);
EXTERN_INTERP_FUNC(lor);
EXTERN_INTERP_FUNC(ixor);
EXTERN_INTERP_FUNC(lxor);
EXTERN_INTERP_FUNC(iinc);
EXTERN_INTERP_FUNC(i2l);
EXTERN_INTERP_FUNC(i2f);
EXTERN_INTERP_FUNC(i2d);
EXTERN_INTERP_FUNC(l2i);
EXTERN_INTERP_FUNC(l2f);
EXTERN_INTERP_FUNC(l2d);
EXTERN_INTERP_FUNC(f2i);
EXTERN_INTERP_FUNC(f2l);
EXTERN_INTERP_FUNC(f2d);
EXTERN_INTERP_FUNC(d2i);
EXTERN_INTERP_FUNC(d2l);
EXTERN_INTERP_FUNC(d2f);
EXTERN_INTERP_FUNC(int2byte);
EXTERN_INTERP_FUNC(int2char);
EXTERN_INTERP_FUNC(int2short);
EXTERN_INTERP_FUNC(lcmp);
EXTERN_INTERP_FUNC(fcmpl);
EXTERN_INTERP_FUNC(fcmpg);
EXTERN_INTERP_FUNC(dcmpl);
EXTERN_INTERP_FUNC(dcmpg);
EXTERN_INTERP_FUNC(ifeq);
EXTERN_INTERP_FUNC(ifne);
EXTERN_INTERP_FUNC(iflt);
EXTERN_INTERP_FUNC(ifge);
EXTERN_INTERP_FUNC(ifgt);
EXTERN_INTERP_FUNC(ifle);
EXTERN_INTERP_FUNC(ificmpeq);
EXTERN_INTERP_FUNC(ificmpne);
EXTERN_INTERP_FUNC(ificmplt);
EXTERN_INTERP_FUNC(ificmpge);
EXTERN_INTERP_FUNC(ificmpgt);
EXTERN_INTERP_FUNC(ificmple);
EXTERN_INTERP_FUNC(ifacmpeq);
EXTERN_INTERP_FUNC(ifacmpne);
EXTERN_INTERP_FUNC(goto);
EXTERN_INTERP_FUNC(jsr);
EXTERN_INTERP_FUNC(ret);
EXTERN_INTERP_FUNC(tableswitch);
EXTERN_INTERP_FUNC(lookupswitch);
EXTERN_INTERP_FUNC(ireturn);
EXTERN_INTERP_FUNC(lreturn);
EXTERN_INTERP_FUNC(freturn);
EXTERN_INTERP_FUNC(dreturn);
EXTERN_INTERP_FUNC(areturn);
EXTERN_INTERP_FUNC(return);
EXTERN_INTERP_FUNC(getstatic);
EXTERN_INTERP_FUNC(putstatic);
EXTERN_INTERP_FUNC(getfield);
EXTERN_INTERP_FUNC(putfield);
EXTERN_INTERP_FUNC(invokevirtual);
EXTERN_INTERP_FUNC(invokenonvirtual);
EXTERN_INTERP_FUNC(invokestatic);
EXTERN_INTERP_FUNC(invokeinterface);
EXTERN_INTERP_FUNC(new);
EXTERN_INTERP_FUNC(newarray);
EXTERN_INTERP_FUNC(anewarray);
EXTERN_INTERP_FUNC(arraylength);
EXTERN_INTERP_FUNC(athrow);
EXTERN_INTERP_FUNC(checkcast);
EXTERN_INTERP_FUNC(instanceof);
EXTERN_INTERP_FUNC(monitorenter);
EXTERN_INTERP_FUNC(monitorexit);
EXTERN_INTERP_FUNC(wide);
EXTERN_INTERP_FUNC(multianewarray);
EXTERN_INTERP_FUNC(ifnull);
EXTERN_INTERP_FUNC(ifnonnull);
EXTERN_INTERP_FUNC(goto_w);
EXTERN_INTERP_FUNC(jsr_w);
EXTERN_INTERP_FUNC(breakpoint);

#endif /* INTERP_H */

==> japhar-0.05.tar.gz: japhar-0.05/include/interp.h <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   interp.h -- java interpreter definitions.
   Created: Chris Toshok <toshok@hungry.com>, 13-Aug-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#ifndef _interp_h
#define _interp_h

#include "config.h"
#include "objects.h"
#include "op_stack.h"
#include "sig.h"
#include "ClazzFile.h"

typedef struct _StackFrame StackFrame;

typedef struct JThreadInfo {
  /* A reference to the java thread which represents
     this native thread. */
  jobject java_thread;

  StackFrame* stack_highwater; /* the highest address possible for this thread's stack. */
  StackFrame* stack_lowwater;  /* the lowest address -- the address where the thread stack
				  is allocated. We use this to check for depth == 0 */
  StackFrame* top_frame;

  OpStack *op_stack; /* this thread's operand stack */

  char *name;
  JavaStackItem return_value;
  jint current_state;  /* whether we're suspended, running, etc, etc. */
} JThreadInfo;

typedef struct {
  char *key;
  char *value;
} propertySpec;

/* possible thread state settings */
#define STATE_RUNNING           0x00
#define STATE_INTERRUPTED       0x01
#define STATE_FINISHED          0x03

/* access macros */
#define STATE(t) (t)->current_state
#define TOPFRAME(t) (t)->top_frame
#define PRIORITY(t) (t)->priority
#define NAME(t) (t)->name

#define FRAME_NATIVE			0x01
#define FRAME_HASCLASSLOADER	0x02

struct _StackFrame {
  int frame_size;
  int depth;

  unsigned char flags;

  MethodStruct *method;
  JThreadInfo *thread_info;
  JNIEnv *jni_env;

  int pc;
  int wide_vindex_set;

  char *className;
  char *methodName;
  char *methodSig;

  JavaStackItem *opstack_top;

  jobject this_pointer;
  JavaStackItem vars[1];
};

/* access macros */

#define PARENT(f) (StackFrame*)(((char*)f) + f->frame_size)
#define ENV(f) (f)->jni_env
#define THREAD_INFO(f) (f)->thread_info
#define STACKDEPTH(f) (f)->depth

#define THISPTR(f) (f)->this_pointer

#define CLASSNAME(f) (f)->className
#define METHODNAME(f) (f)->methodName
#define METHODSIG(f) (f)->methodSig

#define METHOD(f) (f)->method
#define CODE(f) (f)->method->code
#define EXCEPTIONS(f) (f)->method->exceptions
#define NUMEXCEPTIONS(f) (f)->method->num_exception_blocks
#define PC(f) (f)->pc
#define WIDE(f) (f)->wide_vindex_set
#define VARS(f) (f)->vars

#define OPSTACK(f) THREAD_INFO(f)->op_stack
#define OPSTACK_TOP(f) (f)->opstack_top

jboolean setup_stackframes(JThreadInfo *thread_info);
StackFrame *create_frame_for_method(JThreadInfo *thr_info, MethodStruct *info);
void destroy_frame (StackFrame *f);

void fill_local_vars_from_stack (StackFrame *df, /* destination frame */
				 int num_words_needed,
				 int static_p); /* is this a static call? */
void fill_local_vars (StackFrame *df, /* destinatin frame */
		      MethodStruct *method,
		      jvalue *args, /* must match the number in the signature. */
		      jobject obj); /* NULL for a static call. */

StackFrame* push_frame(JThreadInfo *thr_info, int num_vars);
void pop_frame(JThreadInfo *thr_info);
StackFrame *get_frame_parent(StackFrame *f);

void do_native_method_call(JNIEnv *env, MethodStruct *);

void execute_opcode(StackFrame *f, u1 opcode);

jvalue CallJavaMethod(MethodStruct *method, jobject obj, jvalue *args);
jvalue CallStaticJavaMethod(MethodStruct *method, jclass clazz, jvalue *args);

void interp_loop(StackFrame *current_frame);

void set_instance_field(japhar_obj obj, FieldStruct *field, jvalue value);
void set_static_field(ClazzFile *clazz, FieldStruct *field, jvalue value);

void get_instance_field(japhar_obj obj, FieldStruct *field, jvalue *value);
void get_static_field(ClazzFile *clazz, FieldStruct *field, jvalue *value);

#define get_next_u1(f) (CODE(f)[ PC(f) ++ ])

extern void maybe_enter_monitor_for_method(JNIEnv *env,
					   MethodStruct *method,
					   japhar_obj this_obj);
extern void maybe_exit_monitor_for_method(JNIEnv *env,
					  MethodStruct *method,
					  japhar_obj this_obj);
extern void maybe_push_return_value(StackFrame *f);

typedef void (*interp_function)(StackFrame *);

typedef struct opcode_struct {
  char name[26];
  int num_bytes_needed;
  int num_stack_items_before;
  int num_stack_items_after;
  interp_function func;
} opcode;

#endif /* INTERP_H */
